// ----------------------------------------------------------------------------
//
// rt_array.h
// RT++ garbage collected arrays
//
// $Id: rt_array.h,v 1.10 1996/02/28 15:51:55 ws Exp $
// (C) 1996 Wolfgang Schreiner <Wolfgang.Schreiner@risc.uni-linz.ac.at>
//
// ----------------------------------------------------------------------------
#ifndef RT_ARRAY_H
#define RT_ARRAY_H

// ----------------------------------------------------------------------------
//
// array header information
// (gcc does not accept this as a local class in Array<T>)
//
// ----------------------------------------------------------------------------

class ArrayInfo                // the array header
{			         
  public:		         
  int length;                  // number of array elements
};			       
		    	       
class ArrayBlock: public Block // an array memory block
{					    
  public:				    
  ArrayInfo info;			    
};

// ----------------------------------------------------------------------------
//
// array of <T> elements
// an array is a reference to a block of <n> elements of type <T>
//
// ----------------------------------------------------------------------------
template <class T>
class Array
{
  friend class ThreadBagCore;               // scan routine of bags
  protected:
  T *pointer;                               // pointer to first element
  static ArrayBlock *block(T *p);           // memory info from array
  Array(T *p);                              // initialization from pointer
  void _alloc(int n);                       // core allocation
  public:				    
  void init(void);                          // set array reference to empty
  void alloc(int n);                        // allocate array with <n> elems
  Array(void);                              // construct empty array ref
  Array(int n);                             // construct array with <n> elems
  Array(const Array<T> &a);                 // duplicate array reference
  ~Array(void);                             // destroy array reference
  Array<T>& operator = (const Array<T> &a); // assign array references
  int operator == (const Array<T> &a);      // compare array references
  friend int Null(const Array<T> &a);       // true iff reference empty
  static Array<T> Alloc(int n);             // allocate array with <n> elems
  friend int Length(Array<T> a);            // number of array elements
  T& operator [] (int i);                   // indexing operation
  void scan(void);                          // the scan function of this type
};

#endif
// ----------------------------------------------------------------------------
// end of rt_array.h
// ----------------------------------------------------------------------------
// Local Variables: ***
// mode: C++ ***
// End: ***
