// ----------------------------------------------------------------------------
//
// rt_stack.h
// RT++ stack information
//
// $Id: rt_stack.h,v 1.11 1996/03/27 08:17:31 schreine Exp $
// (C) 1996 Wolfgang Schreiner <Wolfgang.Schreiner@risc.uni-linz.ac.at>
//
// ----------------------------------------------------------------------------
#ifndef RT_STACK_H
#define RT_STACK_H

// ----------------------------------------------------------------------------
//
// the "shadow stack" of a thread
// describes the address and type of each heap pointer on the thread stack
//
// ----------------------------------------------------------------------------
class Stack
{
  class Reference
    {
      public:
      void *addr;        // the address of the pointer
      void(*fun)(void*); // the type of the pointer (its scan function)
    };
  Reference *top;                       // the top pointer of the stack
  Reference *shadow;                    // the start address of the stack
  static Reference* alignup(void*);     // upwards alignment of shadow stack
  static Reference* aligndown(void*);   // downwards alignment of shadow stack
  Reference *upper(void);               // upper address for scanning (incl)
  Reference *lower(void);               // lower address for scanning (excl)
  public:
  Stack *next;                          // link to next stack in gc
  int gc;                               // flag for gc
  void init(void);                      // set stack to empty
  void init(void *stack);               // init with stack info
  int null(void);                       // true iff stack not initialized
  void push(void *addr, void(*fun)(void*)); // push reference on stack
  void pop(void *addr);                 // pop reference from stack
  void mark(void);                      // mark all referenced blocks
  void check(void);                     // check for stack overflow
};

#endif
// ----------------------------------------------------------------------------
// end of rt_stack.h
// ----------------------------------------------------------------------------
// Local Variables: ***
// mode: C++ ***
// End: ***
