// ----------------------------------------------------------------------------
//
// rt_stat.h
// RT++ statistics
//
// $Id: rt_stat.h,v 1.10 1996/04/02 09:22:37 schreine Exp $
// (C) 1996 Wolfgang Schreiner <Wolfgang.Schreiner@risc.uni-linz.ac.at>
//
// ---------------------------------------------------------------------------
#ifndef RT_STAT_H
#define RT__STAT_H
 
// ---------------------------------------------------------------------------
//
// argument structure
// contains all parameters that may be configured by the user
//
// ---------------------------------------------------------------------------
class RT_Argument
{
  public:
  int proc;    // number of processes
  int memory;  // size of shared memory
  int heap;    // initial size of garbage-collected heap
  int blocks;  // number of heap blocks per processor
  int stack;   // size of thread stack
  int verbose; // verbosity flag
};

extern RT_Argument rt_argument; // program argument (local process copy)

// ---------------------------------------------------------------------------
//
// result structure
// contains all information returned by the system
//
// ---------------------------------------------------------------------------
class RT_Result
{
  public:
  int ret_code;     // return code
  int time_real;    // real time from program start to program end
  int time_proc;    // process time of current process (user + system)
  int time_all;     // process time of all processes (user + system)
  int gc_number;    // number of garbage collections
  int gc_inc;       // number of garbage collection increments
  int gc_wait;      // synchronization time of garbage collections
  int gc_time;      // execution time of garbage collections
  int gc_blocks;    // number of collected blocks
  int gc_bytes;     // amount of collected bytes
  void init(void);  // initialize structure with current values
  void print(void); // print structure
};

extern RT_Result *rt_result; // pointer to program result (shared)

// ---------------------------------------------------------------------------
//
// statistics collected by the system
// internal use only
//
// ---------------------------------------------------------------------------
class Statistics
{
  friend class RT_Result;
  int verbose;                        // verbosity flag
  int start_time;                     // start time
  int start_clock;                    // start clock
  int start_all;                      // start all time
  int gc_wtime;                       // garbage collection wait time
  int gc_stime;                       // garbage collection start time
  int gc_number;                      // number of garbage collections
  int gc_incs;                        // number of heap increments
  int gc_blocks;                      // number of blocks reclaimed
  int gc_bytes;                       // number of bytes reclaimed
  int gc_wait;                        // garbage collection wait time
  int gc_time;                        // garbage collection execution time
  public:
  Statistics(int v);
  void start(void);                   // start timings
  void gc_trigger(void);              // report trigger of gc
  void gc_start(void);                // report start of gc 
  void gc_free(int minimum);          // report that too little space free
  void gc_size(int minimum);          // report that blocks are too small
  void gc_inc(int size);              // report allocation of additional memory
  void gc_end(int blocks, int bytes); // report end of gc
  void report(void);                  // print report
};

extern Statistics *rt_stat; // pointer to statistics object (shared)

// initialize the statistics object
void stat_init(int verbose);

#endif
// ----------------------------------------------------------------------------
// end of rt_stat.h
// ----------------------------------------------------------------------------
// Local Variables: ***
// mode: C++ ***
// End: ***
