// ----------------------------------------------------------------------------
//
// pointers.cc
// test of RT++ pointers and garbage collection
//
// $Id: pointers.cc,v 1.4 1996/04/02 08:45:14 schreine Exp $
// (C) 1996 Wolfgang Schreiner <Wolfgang.Schreiner@risc.uni-linz.ac.at>
//
// ----------------------------------------------------------------------------

#include <rt++.h>

template <class T>
struct Node
{
  T val;
  Pointer<Node> lson;
  Pointer<Node> rson;
  void init(void)
    {
      void (*f)(T*) = Type<T>::init;
      if (f) f(&val);
      lson.init();
      rson.init();
    }
  void scan(void)
    {
      void (*f)(T*) = Type<T>::scan;
      if (f) f(&val);
      lson.scan();
      rson.scan();
    }
};

template <class T>
struct Tree: public Pointer< Node<T> >
{
  Tree(void): Pointer< Node<T> >()
    {
    }
  Tree(int n): Pointer< Node<T> >(n)
    {
    }
  Tree(Pointer< Node<T> > t): Pointer< Node<T> >(t)
    {
    }
};

Atom(int);
Ref(Node<int>);

Tree<int> create(int n)
{
  Tree<int> t;
  if (n <= 0) return(t);
  t.alloc();
  t->val = n;
  t->lson = create(n-1);
  t->rson = create(n-1);
  return(t);
}

// g++ does not generate this instantiation automatically
template int Null(Pointer< Node<int> >);

int check(Tree<int> t)
{
  if (Null(t)) return(0);
  int d1 = check(t->lson);
  int d2 = check(t->rson);
  if (d1 != d2 || d1+1 != t->val) rt_abort("depth error");
  return(t->val);
}

int pointers(int, char *[])
{
  Tree<int> s;
  for (int i = 0; i < 200 ; i++)
    {
      Tree<int> t = create(rand()%12);
      check(s);
      s = t;
    }
  return(0);
}

void init(RT_Argument *rt_arg, int argc, char *argv[]);

int main(int argc, char *argv[]) 
{
  RT_Argument rt_arg;
  init(&rt_arg, argc, argv);
  exit(rt_main(pointers, 1, argv, rt_arg, 0));
}

// ----------------------------------------------------------------------------
// end of pointers.cc
// ----------------------------------------------------------------------------
// Local Variables: ***
// mode: C++ ***
// End: ***
